@@ -280,8 +280,9 @@ If CH is the character \"f\" for \"conditions\", then the string
        (setq header-line-format
              (nethack-status-string nethack-status-header-line-format))))
     (:mode-line
-     (setq mode-line-format
-           (nethack-status-string nethack-status-mode-line-format)))
+     (with-current-buffer nethack-map-buffer
+       (setq mode-line-format
+             (nethack-status-string nethack-status-mode-line-format))))
     (:map
      (with-current-buffer nethack-map-buffer
        (let ((p (next-single-property-change (point-min) 'nethack-status))
@@ -291,11 +292,12 @@ If CH is the character \"f\" for \"conditions\", then the string
          (nethack-with-point
           (goto-char (point-max))
           (insert (propertize (nethack-status-string nethack-status-buffer-format) 'nethack-status t))))))
-    (t
+    (:buffer
      (with-current-buffer nethack-status-buffer
        (let ((inhibit-read-only t))
          (erase-buffer)
-         (insert (nethack-status-string nethack-status-buffer-format)))))))
+         (insert (nethack-status-string nethack-status-buffer-format)))))
+    (t (setq nethack-status-style :buffer) (warn "nethack-status-style: t has been deprecated in favor of :buffer"))))
 
 
 
@@ -438,6 +440,7 @@ accessed 2021-04-23.")
                                                   ('metalivore `(,all-categories))
                                                   (_ '(("Comestibles"))))))
                                      ("write with" . (("Coins" "Weapons" "Miscellaneous" "Wands" "Tools" "Gems/Stones")))
+                                     ("write on" . (("Scrolls" "Spellbooks")))
                                      ("wield" . ((,@all-categories "Miscellaneous") . ("being worn" "wielded")))
                                      ("read" . (("Scrolls" "Spellbooks")))
                                      ("ready" . ((,all-categories) . ("being worn")))
@@ -471,6 +474,7 @@ accessed 2021-04-23.")
                          ;; yes, if you name a weapon "(wielded)" or some dumb shit like that, this will break.
                          (name-matcher (lambda (s) (string-match-p (format "(%s)$" (regexp-quote s)) name))))
                     (and (member category (car category-predicates))
+                         (or (not (string= action "write on")) (string-match-p "\\(?:unlabeled scroll\\|plain spellbook\\)" name))
                          (if (eq (cadr category-predicates) t) ;; invert match
                              (cl-some name-matcher (cddr category-predicates))
                            (not (cl-some name-matcher (cdr category-predicates)))))))
@@ -486,6 +490,7 @@ accessed 2021-04-23.")
                          (category (cddr i))
                          (name-matcher (lambda (s) (string-match-p (format "(%s)$" (regexp-quote s)) name))))
                     (when (and (member category (car category-predicates))
+                         (or (not (string= action "write on")) (string-match-p "\\(?:unlabeled scroll\\|plain spellbook\\)" name))
                          (if (eq (cadr category-predicates) t) ;; invert match
                              (cl-some name-matcher (cddr category-predicates))
                            (not (cl-some name-matcher (cdr category-predicates)))))
@@ -538,6 +543,10 @@ accessed 2021-04-23.")
                           key))))
     (message ques)))
 
+(define-advice nethack-nhapi-yn-function (:after (ques _choices _default))
+  (when (string= ques "Do you want to keep the save file?")
+    (nethack-nhapi-restore-window-configuration)))
+
 (defun nethack-nhapi-ask-direction (prompt)
   "Prompt the user for a direction."
   (if nethack-proc
@@ -565,11 +574,20 @@ accessed 2021-04-23.")
       (nethack-send (nethack-read-line "Who are you? "))
     (message "Who are you? ")))
 
-(defun nethack-nhapi-getlin (ques)
+(defun nethack-nhapi-getlin (ques &optional initial)
   (if nethack-proc
+      (if (and nethack-want-completing-read
+               (string-match "\\(What do you want to \\(dip \\(.*\\) into\\|.*\\)\\? \\)\\[" ques))
+            (let* ((prompt (match-string 1 ques))
+                   (action (if (match-string 3 ques) "dip into" (match-string 2 ques)))
+                   (ret (condition-case _ (completing-read prompt (nethack-completing-read-filter action)) (quit "\0"))))
+              (while (not (or (assoc-string ret nethack--inventory) (string= ret "\0")))
+                (run-with-timer 0.01 nil (lambda () (minibuffer-message "match required")))
+                (setq ret (condition-case _ (completing-read prompt (nethack-completing-read-filter action)) (quit "\0"))))
+                (nethack-send (string-to-char ret)))
       (nethack-send (condition-case nil
-                        (nethack-read-line (concat ques " "))
-                      (quit "")))
+                        (nethack-read-line (concat ques " ") initial)
+                      (quit ""))))
     (message ques)))
 
 (defun nethack-nhapi-get-ext-cmd (cmd-alist)
@@ -642,9 +660,7 @@ Do not edit the value of this variable.  Instead, change the value of
   "If non-nil, at the next command prompt, update the menu.")
 (defvar nethack--inventory nil)
 
-(defun nethack-nhapi-update-inventory ()
-  (setq nethack--inventory nil)
-  (setq nethack-inventory-need-update t))
+(defun nethack-nhapi-update-inventory ())
 
 (defun nethack-nhapi-doprev-message ()
   (save-selected-window
@@ -665,6 +681,8 @@ Do not edit the value of this variable.  Instead, change the value of
   (setq nethack-directory (file-name-directory executable))
   (when (and (nethack-options-set-p 'tiled_map) (null nethack-use-tiles))
     (message "You have OPTIONS=tiled_map set in your nethackrc; consider setting nethack-use-tiles"))
+  (when (and nethack-want-completing-read (not (nethack-options-set-p 'perm_invent)))
+    (message "nethack-want-completing-read does not function properly without OPTIONS=perm_invent"))
   (setq nethack-nhapi-print-glyph--previous-ch t)
   (setq nethack--inventory nil)
   ;; clean up old buffers
@@ -1031,6 +1049,9 @@ displayed."
 accelerator that will be used in unassigned menus.")
 
 (defun nethack-nhapi-start-menu (menuid)
+  (when (= menuid 4)
+    (setq nethack--inventory nil)
+    (setq nethack-inventory-need-update t))
   (with-current-buffer (nethack-menu-buffer menuid)
     (let ((inhibit-read-only t))
       (erase-buffer)
@@ -1069,10 +1090,12 @@ buffer."
       (when (nethack-options-set-p 'menucolors)
         (nethack-options-highlight-menu))
       (when nethack-inventory-need-update
-        (if (= accelerator -1)
-            (push `(,str) nethack--inventory)
-          (let ((start (save-excursion (search-backward str))))
-            (push (cons (char-to-string accelerator) (buffer-substring start (+ start (length str)))) (cdar nethack--inventory)))))
+        (condition-case nil
+            (if (= accelerator -1)
+                (push `(,str) nethack--inventory)
+              (let ((start (save-excursion (search-backward str))))
+                (push (cons (char-to-string accelerator) (buffer-substring start (+ start (length str)))) (cdar nethack--inventory))))
+          (error (setq nethack-inventory-need-update nil))))
       (run-hooks 'nethack-add-menu-hook))))
 
 ;; FIXME: xemacs propertize bug here
@@ -1092,7 +1115,7 @@ Saves the current window configuration so that it can be restored when
 the menu is dismissed."
   (let ((buffer (nethack-menu-buffer menuid)))
     (unless buffer (error "No such menuid: %d" menuid))
-    (if nethack-inventory-need-update
+    (when nethack-inventory-need-update
         (progn
           (setq nethack-inventory-need-update nil)
           ;; transpose from '((CATEGORY (ACCEL . NAME))) to '((ACCEL NAME . CATEGORY))
@@ -1103,7 +1126,7 @@ the menu is dismissed."
                                       `(,(car item) ,(cdr item) . ,(car category)))
                                     (cdr category)))
                           nethack--inventory)))
-          (nethack-send nil))
+          (nethack-send nil)))
       (progn
         (unless nethack-active-menu-buffer
           (setq nethack-window-configuration (current-window-configuration)))
@@ -1126,7 +1149,7 @@ the menu is dismissed."
         (nethack-menu-mode how)
         (goto-char (point-min))
         (message "Displaying menu")
-        (setq nethack-active-menu-buffer buffer)))))
+        (setq nethack-active-menu-buffer buffer))))
 
 (defmacro nethack-nhapi--let*-if (condition then-bindings else-bindings &rest body)
   `(if ,condition
@@ -1146,13 +1169,11 @@ the menu is dismissed."
 
   (nethack-nhapi--let*-if (and nethack-use-tiles (display-graphic-p))
                           ((_ (selected-window)) ; layout similar to x11 port
-                           (w-status (split-window-vertically (floor (* 0.95 (window-body-height)))))
                            (w-map (split-window-vertically (floor (* 0.55 (window-body-height)))))
                            (w-message (selected-window))
                            (w-inventory (when (nethack-options-set-p "perm_invent") (progn (split-window-horizontally (floor (* 0.6 (window-body-width))))))))
                           ((w-left (selected-window)) ; layout similar to curses port
                            (w-right (split-window-horizontally (floor (* 0.6 (window-width)))))
-                           (w-status (split-window-vertically (floor (* 0.95 (window-body-height)))))
                            (w-inventory (when (nethack-options-set-p "perm_invent") (progn (select-window w-right) (split-window-vertically (floor (* 0.4 (window-body-height)))))))
                            (w-map w-left)
                            (w-message w-right))
@@ -1172,13 +1193,14 @@ the menu is dismissed."
                           (set-window-dedicated-p w-map nil)
                           (set-window-buffer w-message nethack-message-buffer)
                           (set-window-dedicated-p w-message nil)
-                          (set-window-buffer w-status nethack-status-buffer)
-                          (set-window-dedicated-p w-status t)
-                          (window-preserve-size w-status nil t)
-
-                          (with-current-buffer nethack-status-buffer
-                            (setq mode-line-format nil))
 
+                          (when (eq nethack-status-style :buffer)
+                            (let ((w-status (split-window-vertically (- nethack-status-window-height) w-map)))
+                              (set-window-buffer w-status nethack-status-buffer)
+                              (set-window-dedicated-p w-status t)
+                              (window-preserve-size w-status nil t)
+                              (with-current-buffer nethack-status-buffer
+                                (setq mode-line-format nil))))
                           (select-window w-map)))
 
 
