@@ -5,8 +5,8 @@
 ;; Author: Ryan Yeske <rcyeske@vcn.bc.ca>
 ;; Maintainer: George Huebner <george@feyor.sh>
 ;; Created: Sat Mar 18 11:31:52 2000
-;; Package-Version: 20251025.1205
-;; Package-Revision: c3d4b8980fa8
+;; Package-Version: 20251213.1310
+;; Package-Revision: 475d6113aa8a
 ;; Keywords: games
 ;; URL: https://github.com/Feyorsh/nethack-el
 
@@ -96,11 +96,11 @@
   :type '(string)
   :group 'nethack)
 
-(defcustom nethack-status-style t
+(defcustom nethack-status-style :buffer
   "Decides how the status will be displayed.
-Valid values are :map, :header-line, :mode-line, or t."
+Valid values are :map, :header-line, :mode-line, or :buffer."
   :type '(symbol)
-  :options '(:map :mode-line :header-line t)
+  :options '(:map :mode-line :header-line :buffer)
   :group 'nethack)
 
 (defcustom nethack-purge-buffers t
